/***************************************************************************
                          gifdraw.cpp  -  description
                             -------------------
    begin                : Fri Jun 23 2000
    copyright            : (C) 2000 by Helmut Fahrion
    email                : hefa@bitctrl.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "gifblocks.h"


// *** User berladefunktion  
// Userinputfunction, kann belibig berladen werden, default wartet 1s
void
imagedesc::userinput (char verbose)
{
  //QString msg;
  //msg.sprintf("User Input!");  
  //QMessageBox::message("Eingabe", msg);
  //nicht so toll, da die Box neue zeichenevents auslst, und wieder Boxen ffnet ! 
  usleep (1000000L);

  if (verbose)
    cout << "User Input, wait 1 sec!\n";
}

// *** User berladefunktion 
// Zeichnet Hintergrund fr Transparente Images
void
imagedesc::userbg (void *wdg, unsigned char mode)
{
  unsigned char g = 15, c = 200;

  qbr.setStyle (SolidPattern);

  // Defaultbg ist Einfarbib
  if (mode)
    {
      qbr.setColor (QColor (c, c, c));
      ((QPainter *) wdg)->fillRect (l, t, w, h, qbr);
    }
  else
    {
      qbr.setColor (QColor (c, c, c));
      for (int i = g; i < w; i += g * 2)
	for (int j = g; j < h; j += g * 2)
	  ((QPainter *) wdg)->fillRect (i, j, g, g, qbr);

      c -= 30;
      qbr.setColor (QColor (c, c, c));
      for (int i = 0; i < w; i += g * 2)
	for (int j = g; j < h; j += g * 2)
	  ((QPainter *) wdg)->fillRect (i, j, g, g, qbr);

      c -= 30;
      qbr.setColor (QColor (c, c, c));
      for (int i = g; i < w; i += g * 2)
	for (int j = 0; j < h; j += g * 2)
	  ((QPainter *) wdg)->fillRect (i, j, g, g, qbr);

      c -= 30;
      qbr.setColor (QColor (c, c, c));
      for (int i = 0; i < w; i += g * 2)
	for (int j = 0; j < h; j += g * 2)
	  ((QPainter *) wdg)->fillRect (i, j, g, g, qbr);

    }
}

// ********* Image Description ******************* 
void
imagedesc::drawline (int y, int n, unsigned char tf, unsigned char ti,
		     void *widget)
{
  int pidx, x, xv, yt, nw = n * w;
  unsigned char cidx;

  for (x = 0; x < w; x++)
    {
      pidx = 0;
      cidx = pix[nw + x].p;

      color c ((cmap) ? cmap->getcolor (cidx) : scrblk->cmap->
	       getcolor (cidx));
      ((QPainter *) widget)->setPen (QColor (c.r, c.g, c.b));

      for (xv = x + 1; xv < w; xv++)
	{
	  if (pix[nw + xv].p == cidx)
	    pidx++, x++;
	  else
	    break;
	}

      xv = x + l;
      yt = y + t;
      if (tf)
	{
	  if (cidx != ti)
	    {
	      if (pidx)
		((QPainter *) widget)->drawLine (xv - pidx, yt, xv, yt);
	      else
		((QPainter *) widget)->drawPoint (xv, yt);
	    }
	}
      else
	{
	  if (pidx)
	    ((QPainter *) widget)->drawLine (xv - pidx, yt, xv, yt);
	  else
	    ((QPainter *) widget)->drawPoint (xv, yt);
	}
    }
}

void
imagedesc::draw (void *widget, void *win)
{
  int y, n = 0;

  char tf;
  unsigned char ti;
  color c;

  // Optim statisch
  //QPixmap qpixmap(w, h);

  qbr.setStyle (SolidPattern);

  if (!pix)
    {
      cerr << "Keine Pixel zum zeichnen!\n";
      return;
    }

  // Draw Image  
  // Transparency Flag
  // Transparency Index
  if (gcontrol)
    tf = gcontrol->tc, ti = gcontrol->tci;
  else
    tf = ti = 0;

  if (i)
    {
      // The rows of an Interlaced images are arranged in the following order:
      // Optimierung, drawline knnte eine Pixmap mit der Hhe 1 zeichnen
      for (n = y = 0; y < h; y += 8, n++)
	drawline (y, n, tf, ti, widget);
      for (y = 4; y < h; y += 8, n++)
	drawline (y, n, tf, ti, widget);
      for (y = 2; y < h; y += 4, n++)
	drawline (y, n, tf, ti, widget);
      for (y = 1; y < h; y += 2, n++)
	drawline (y, n, tf, ti, widget);
    }
  else
    // Optimierung, als Image zeichnen
    {
      for (y = 0; y < h; y++)
	drawline (y, y, tf, ti, widget);

      /*
         ((QPainter *) widget)->end();   
         ((QPainter *) widget)->begin( &qpixmap );

         for (y = 0; y < h; y++)
         {
         drawline (y, y, tf, ti, widget);

         } 

         bitBlt( (QWidget*)win, 0, 0, &qpixmap);

         ((QPainter *) widget)->end();        
         ((QPainter *) widget)->begin( (QWidget*)win );
       */
    }

  // vor Disposal mu geflusht werden, sonst knnte es sein das manche Grafik
  // unvollstndig wird    
  ((QPainter *) widget)->flush ();

  if (gcontrol)
    // Disposal Method
    if (gcontrol->dm)		//No disposal specified. The decoder is not required to take any action.
      {
	// User Input Flag  
	if (gcontrol->ui)
	  {
	    userinput (verb);
	  }
	else
	  {
	    // Delay Time
	    if (gcontrol->dt)
	      {
		//cout << "sleep: " << gb->dt << "\n";
		usleep (gcontrol->dt * 10000L);
	      }
	  }

	switch (gcontrol->dm)
	  {
	  case 1:
	    break;		// Do not dispose. The graphic is to be left in place.
	    // Restore to background color. The area used by the 
	    // graphic must be restored to the background color.
	  case 2:
	    if (tf)
	      {
		// Altes BG Image widerherstellen oder Farbe
		userbg (widget, 0);
	      }
	    else
	      {
		// globale Farbtabelle
		c = scrblk->cmap->getcolor (scrblk->sbc);
		qbr.setColor (QColor (c.r, c.g, c.b));
		((QPainter *) widget)->fillRect (l, t, w, h, qbr);
	      }

	    ((QPainter *) widget)->flush ();
	    break;
	    // Restore to previous. The decoder is required to 
	    // restore the area overwritten by the graphic with
	    // what was there prior to rendering the graphic.
	  case 3:
	    if (prev)
	      {
		if (prev->gcontrol)
		  {
		    int odm;
		    graphicsblock *gbv;
		    gbv = prev->gcontrol;
		    odm = gbv->dm;
		    gbv->dm = 0;	// mus 0 gesetzt werden sonst kann es eine Schleife geben
		    prev->draw (widget, win);
		    gbv->dm = odm;
		  }
		else
		  {
		    prev->draw (widget, win);
		  }
	      }
	    break;
	  }
      }
}

// ********** Text Block ************
void
pstrblock::draw (void *widget, void *win)
{
  unsigned int x, xl, yl, px, py;
  QString qstr;


  QFont f ("Fixed", bblk->cch, bblk->ccw, FALSE);	// laut Standert ein "Fixed" Font 

  if (ptstr)
    {
      ((QPainter *) widget)->setFont (f);
      ((QPainter *) widget)->setPen (QColor (bblk->fgc.r, bblk->fgc.g, bblk->fgc.b));

      qbr.setStyle (SolidPattern);
      qbr.setColor (QColor (bblk->bgc.r, bblk->bgc.g, bblk->bgc.b));

      for (yl = x = xl = 0; x < strlen (ptstr); x++, xl++)
	{
	  // Zeilenschaltung
	  if (x >= 1 && ptstr[x - 1] == '\n')
	    {
	      yl++;
	      xl = 0;
	    }

	  if (xl <= bblk->tgw && yl <= bblk->tgh)
	    {
	      px = bblk->tglp + xl * bblk->ccw;
	      py = bblk->tgtp + yl * bblk->cch;

	      qstr.sprintf ("%c", ptstr[x]);
	      // Hintergrund
	      ((QPainter *) widget)->fillRect (px, py, bblk->ccw, bblk->cch, qbr);
	      // der Text  -2 ist Kosmetik
	      ((QPainter *) widget)->drawText (px, bblk->cch + py - 2, qstr, 1);
	    }
	}
    }
}
